// -*- C++ -*-
//
// dialogs.h
//
// Copyright (c) 2007 Sergio Perticone.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
// EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef GX_DIALOGS
#define GX_DIALOGS

#include "image.h"
#include <gtkmm/adjustment.h>
#include <gtkmm/box.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/combobox.h>
#include <gtkmm/button.h>
#include <gtkmm/dialog.h>
#include <gtkmm/frame.h>
#include <gtkmm/label.h>
#include <gtkmm/liststore.h>
#include <gtkmm/notebook.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/scale.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/separator.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/table.h>
#include <gtkmm/textview.h>
#include <libgnomevfsmm/transfer.h>

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

namespace Gx { namespace Dialog {

	class Header : public Gtk::VBox {
	public:
		Header(const Glib::ustring& text,
		       const Gx::Image& image);

		virtual ~Header() { }

	protected:
		Gtk::VBox       vbox;
		Gtk::HBox       hbox;
		Gtk::Image     thumb;
		Gtk::Label    header;
		Gtk::Label      name;
		Gtk::HSeparator  sep;
	};

	class Info : public Gtk::Dialog {
		typedef std::map<const char*, Gtk::Entry*> EntryMap;
		typedef std::map<const char*, Gtk::Label*> LabelMap;

	public:
		Info(const Gx::Image&);
		virtual ~Info() { }

	protected:
		Header           header;
		Gtk::Notebook        nb;

		Gtk::Table        table;
		EntryMap          entry;
		LabelMap          label;

		Gtk::Button       close;

		Gtk::VBox         vbox0;

#if HAVE_EXIF
		
#else
		Gtk::TextView                  textv;
		Glib::RefPtr<Gtk::TextBuffer> buffer;
#endif
	};

	class Saturate : public Gtk::Dialog {
	public:
		Saturate(const Gx::Image&);
		virtual ~Saturate() { }

		virtual double    get_value  () const; // inline

	protected:
		virtual void    fit_preview  ();
		virtual void update_preview  ();
		virtual void       on_reset  (); // inline

		Header             header;
		Gtk::Adjustment       adj;
		Gtk::HScale         scale;
		Gtk::Image          image;
		const Gx::Image*   pImage;
		Gtk::HBox            hbox;

		Glib::RefPtr<Gdk::Pixbuf>        ref_img;
		Glib::RefPtr<Gdk::Pixbuf> preview_pixbuf;

		Gtk::Button   apply;
		Gtk::Button  cancel;
		Gtk::Button   reset;
	};

	class Scale : public Gtk::Dialog {
	public:
		Scale(Gx::Image&);
		virtual ~Scale() { }

	protected:
		virtual void            on_reset  ();
		virtual void    on_width_changed  ();
		virtual void   on_height_changed  ();
		virtual void            on_scale  (Gx::Image*);
		virtual void       update_l_size  (int  width, 
						   int height);

		class ModelColumns : public Gtk::TreeModel::ColumnRecord {
		public:
			ModelColumns() { add(interp); }

			Gtk::TreeModelColumn<Glib::ustring>  interp;
		};

	private:
		int                 img_width;
		int                img_height;

		Header                 header;
		
		Gtk::Button                ok;
		Gtk::Button            cancel;
		Gtk::Button             reset;
		Gtk::CheckButton        block;

		Gtk::Label             l_file;
		Gtk::Label        l_header_sz;
		Gtk::Label            l_width;
		Gtk::Label           l_height;
		Gtk::Label             l_size;
		Gtk::Label      l_header_qual;
		Gtk::Label           l_interp;

		Gtk::HBox               hbox0;
		Gtk::HBox               hbox1;
		Gtk::HBox               hbox2;
		Gtk::HBox            hbox_but;

		Gtk::Adjustment      adj_width;
		Gtk::Adjustment     adj_height;

		Gtk::SpinButton       s_width;
		Gtk::SpinButton      s_height;

		// fucking combo box:
		Gtk::ComboBox                    combo;		
		ModelColumns                   columns;
		Glib::RefPtr<Gtk::ListStore> ref_model;
	};

	class Supported : public Gtk::Dialog {
	public:
		Supported();
		virtual ~Supported() { }

	protected:
		class TypeColumns : public Gtk::TreeModel::ColumnRecord {
		public:
			TypeColumns() 
			{ 
				add(descr);
				add(name); 
				add(writable); 
				add(not_writable); 
			}

			Gtk::TreeModelColumn<Glib::ustring> descr;
			Gtk::TreeModelColumn<Glib::ustring> name;
			Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > writable;
			Gtk::TreeModelColumn<bool> not_writable; // hidden column
		};

		TypeColumns                    columns;

		Gtk::TreeView                 treeview;
		Glib::RefPtr<Gtk::ListStore>  refModel;

		Gtk::ScrolledWindow         scroll_win;
		Gtk::Button                      close;
	};

	// inline functions:

	inline double Saturate::get_value() const
	{
		return adj.get_value();
	}

	inline void Saturate::on_reset()
	{
		adj.set_value(1.0);
		apply.set_sensitive(false);
	}

} } // Gx::Dialog

#endif // GX_DIALOGS
